#
# Module responsible for verifying user permissions before controller actions are executed.
#
module PermissionsChecker
  extend ActiveSupport::Concern

  # The `included` block is executed when this module is included in a controller.
  # Here, a `before_action` is defined to execute `check_user_permissions` before each controller action.
  included do
    before_action :check_user_permissions
  end

  # Method to handle the response when a user does not have the necessary permissions.
  # This method manages different types of responses depending on the request format.
  def insufficient_user_role()
    respond_to do |format|
      format.html { render template: 'errors/forbidden', alert: privileges_error }
      format.json { render json: { error: privileges_error }, status: :unauthorized }
      format.js
      format.any  { head 403 }
    end
  end

  private

  # Method that returns a generic error message for users with insufficient permissions.
  # This message is used in the responses to users.
  def privileges_error
    "You don't have enough privileges to do that."
  end

  # Main method that checks if the current user has the necessary permissions to perform an action.
  def check_user_permissions
    # Gets the current user.
    user = current_user

    # If no user is logged in, show the insufficient permissions message.
    return insufficient_user_role unless user

    # Gets the name of the current action and converts it to a symbol.
    action = action_name.to_sym
    # Gets the name of the current controller and converts it to a symbol in snake_case format.
    module_name = controller_name.underscore.to_sym

    # Verifies if the user has permissions for the specified module and action.
    # If the user does not have permissions, show the insufficient permissions message.
    unless user.can_in_module?(action, module_name)
      insufficient_user_role
    end
  rescue StandardError => e
    # Captures and logs any unexpected error that occurs while checking permissions.
    Rails.logger.error "Error checking permissions: #{e.class} - #{e.message}"
    # In case of an error, respond with the insufficient permissions message.
    insufficient_user_role
  end
end
