#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class Permissions
  # Define the controllers for which we want to control permissions.
  CONTROLLERS = %i[
    audits model_sources managers integrations mitre
    redborder_objects object_types playbook_templates incidents
    threat_intel_sources playbook_assignment_rules
  ].freeze

  # Define access permissions by role for each controller explicitly.
  # Each role is assigned specific permissions based on allowed actions:
  # - create: create new resources.
  # - remove: delete existing resources.
  # - edit: modify existing resources.
  # - search: search for resources.
  # - view: view resources.
  ROLE_PERMISSIONS = {
    # Permissions for administrators (they have full access to all controllers).
    administrator: CONTROLLERS.each_with_object({}) do |controller, hash|
      hash[controller] = { create: true, remove: true, edit: true, search: true, view: true }
    end,
    incident_responder: {
      audits: { create: false, remove: false, edit: false, search: false, view: false },
      model_sources: { create: false, remove: false, edit: false, search: false, view: false },
      managers: { create: false, remove: false, edit: false, search: false, view: false },
      integrations: { create: false, remove: false, edit: false, search: false, view: false },
      mitre: { create: false, remove: false, edit: false, search: true, view: true },
      redborder_objects: { create: true, remove: false, edit: true, search: true, view: true },
      object_types: { create: false, remove: false, edit: false, search: false, view: false },
      playbook_templates: { create: false, remove: false, edit: false, search: true, view: true },
      incidents: { create: true, remove: true, edit: true, search: true, view: true },
      threat_intel_sources: { create: false, remove: false, edit: false, search: false, view: false },
      playbook_assignment_rules: { create: false, remove: false, edit: false, search: true, view: true }
    },
    security_analyst: {
      audits: { create: false, remove: false, edit: false, search: true, view: true },
      model_sources: { create: false, remove: false, edit: false, search: false, view: false },
      managers: { create: false, remove: false, edit: false, search: false, view: false },
      integrations: { create: true, remove: true, edit: true, search: true, view: true },
      mitre: { create: true, remove: true, edit: true, search: true, view: true },
      redborder_objects: { create: true, remove: true, edit: true, search: true, view: true },
      object_types: { create: true, remove: true, edit: true, search: true, view: true },
      playbook_templates: { create: true, remove: true, edit: true, search: true, view: true },
      incidents: { create: true, remove: true, edit: true, search: true, view: true },
      threat_intel_sources: { create: true, remove: false, edit: false, search: true, view: true },
      playbook_assignment_rules: { create: true, remove: true, edit: true, search: true, view: true }
    },
    viewer: {
      audits: { create: false, remove: false, edit: false, search: false, view: false },
      model_sources: { create: false, remove: false, edit: false, search: false, view: false },
      managers: { create: false, remove: false, edit: false, search: false, view: false },
      correlation_engine_rules: { create: false, remove: false, edit: false, search: false, view: false },
      integrations: { create: false, remove: false, edit: false, search: false, view: false },
      redborder_objects: { create: true, remove: true, edit: true, search: true, view: true },
      object_types: { create: false, remove: false, edit: false, search: false, view: false },
      threat_intel_sources: { create: false, remove: false, edit: false, search: false, view: false },
      playbook_assignment_rules: { create: false, remove: false, edit: false, search: false, view: false }
    }
  }.freeze

  # Initializes the object with the user's role.
  # If the role is not defined, the default role (:viewer) is assigned.
  def initialize(role)
    @role = role ? role.to_sym : :viewer
    @permissions = ROLE_PERMISSIONS.fetch(@role, default_permissions_set)
  end

  # Returns the permissions for a specific controller and, optionally, a specific action.
  def permissions_for(controller_name, action_name = nil)
    # Get the controller permissions or return default permissions if the controller is not defined.
    controller_permissions = @permissions[controller_name] || default_permissions

    # If an action is specified, return the corresponding permission.
    action_name ? controller_permissions[action_type(action_name.to_s)] : controller_permissions
  end

  private

  # Default permissions for actions when not explicitly defined.
  def default_permissions
    { create: false, remove: false, edit: false, search: true, view: true }
  end

  # Creates a default set of permissions for all controllers.
  def default_permissions_set
    CONTROLLERS.each_with_object({}) { |controller, hash| hash[controller] = default_permissions }
  end

  # Determines the action type based on the action name.
  def action_type(action_name)
    action_map = {
      # create
      create: %w[new create load_form_settings common_elements basic_rule_2 create_topic_filters_and_selects create_basic_rule_2 import_tree download_s3 add restore download import import_file create_clone set_as_default new_task_template create_task_template integration_service_actions bulk_create_links new_user create_note create_response_action_note create_response_action_execute generate_ai_reports check_model_job_status force_update check_update create_incident_report create_incident_historical show_page getaddrinfo],
      # edit
      edit: %w[edit update apply assign_rule_to_mitre assign_tactic_to_rule deassign_tactic_from_rule deassign_rule_from_mitre json_mitre_techniques auto_assign_job update_namespace_rules save_cluster_synchronization save_virtual_ip_addresses save_table_data save_rules edit_task_template update_task_template edit_description bulk_update_status assign_user unassign_user new_note new_response_action_note new_response_action_execute edit_note update_note edit_report_modal update_report_section set_enabled update_priority],
      # remove
      remove: %w[destroy remove destroy_task_template close bulk_delete unlink destroy_note],
      # view
      view: %w[show index historical_nodes namespaces load_more_namespaces namespace_rules cluster_synchronization virtual_ip_addresses cluster_status cluster_status_small export_tree view_mitre_matrix set_matrix_custom_time reassign_partitions up_pcp recovery_pcp change_session_tab export next_phase previous_phase sidebar_linked_incidents_content sidebar_observable_content sidebar_content assets_sidebar statistics check_ai_job_status get_incidents_widget_data],
      # search
      search: %w[search search_user reset_filters worklog_search detection_search]
    }

    # Find the action type in the map.
    action_map.each do |action_type, actions|
      return action_type if actions.include?(action_name)
    end

    # If the action name is not found, raise an exception.
    raise ArgumentError, "Unknown action name: #{action_name}"
  end
end
